@@ -7,16 +7,26 @@ import argparse
 from pathlib import Path
 from datetime import datetime
 from collections import Counter
+from operator import itemgetter
 
 import torch
+from torchvision import transforms
 from torchvision.datasets import ImageFolder
 
+import numpy as np
+from tqdm import tqdm
 from PIL import Image, ImageEnhance
 
 from tabulate import tabulate
 
 class VisionDataset:
 
+    transformTorchVision = transforms.Compose([        
+      transforms.Resize((224,224), interpolation=Image.NEAREST),
+      transforms.RandomHorizontalFlip(),
+      transforms.ToTensor(),
+    ])
+
     """
     🧬 Apply data augmentation on the input image
     Source: https://medium.com/lunit/photometric-data-augmentation-in-projection-radiography-bed3ae9f55c3
@@ -39,40 +49,25 @@ class VisionDataset:
     ⚖️ Balance the dataset according to the less represented label
     """
     @staticmethod
-    def __balance(train_ds, train_classes):
+    def __balance(dataset, ct):
+
+        ct_classes = [ct[a] for a in sorted(ct)]
 
         # Get the less represented label in train
-        less_represented_train = min(train_classes)
+        less_represented_train = min(ct_classes)
 
-        labels = {}
+        labels_cpt = {}
+        for label in list(ct.keys()):
+            if label not in labels_cpt:
+                labels_cpt[label] = 0
 
-        # For each image
-        for img, label in train_ds:
+        indices = []
+        for i, label in enumerate(dataset.targets):            
+            if labels_cpt[label] < less_represented_train:
+                labels_cpt[label] += 1
+                indices.append(i)
 
-            # Create the label array if isn't
-            if label not in labels:
-                labels[label] = []
-            
-            # Add the image to the array
-            labels[label].append(img)
-            
-            # New dataset
-            balanced_ds = []
-
-            # For each label
-            for label in labels:
-
-                # Get images
-                imgs = labels[label]
-
-                # For each image
-                for img in imgs[0:less_represented_train]:
-
-                    # Create a tuple: image and label 
-                    t = (img, label)
-
-                    # Add it
-                    balanced_ds.append(t)
+        balanced_ds = torch.utils.data.Subset(dataset, indices)
 
         print("The less represented label in train as " + str(less_represented_train) + " occurrences")
         print("Size of train after balancing is " + str(len(balanced_ds)))
@@ -86,6 +81,16 @@ class VisionDataset:
 
         print("Split Datasets...")
         
+        # If balanced is enabled
+        if balanced == True:
+    
+            print("Balance train dataset...")
+
+            ct = Counter(dataset.targets)
+
+            # Balance the train dataset
+            dataset = VisionDataset.__balance(dataset, ct)
+
         indices = torch.randperm(len(dataset)).tolist()
 
         # Index of the validation corpora
@@ -93,18 +98,7 @@ class VisionDataset:
 
         # TRAIN
         train_ds = torch.utils.data.Subset(dataset, indices[:train_index])
-        ct_train = Counter([label for _, label in train_ds])
-        train_classes = [ct_train[a] for a in sorted(ct_train)]
-        
-        # If balanced is enabled
-        if balanced == True:
-
-            # Balance the train dataset
-            train_ds = VisionDataset.__balance(train_ds, train_classes)
-            
-            # Compute again the stats
-            ct_train = Counter([label for _, label in train_ds])
-            train_classes = [ct_train[a] for a in sorted(ct_train)]
+        print("train_ds: ", len(train_ds))
 
         # If data augmentation is enabled
         if augmentation == True:
@@ -124,8 +118,12 @@ class VisionDataset:
 
         # TEST
         test_ds = torch.utils.data.Subset(dataset, indices[train_index:])
-        ct_test = Counter([label for _, label in test_ds])
-        test_classes = [ct_test[a] for a in sorted(ct_test)]
+        
+        ct_train = Counter(list(map(itemgetter(1), train_ds)))
+        train_classes = [ct_train[int(a)] for a in list(id2label.keys())]
+
+        ct_test = Counter(list(map(itemgetter(1), test_ds)))
+        test_classes = [ct_test[int(a)] for a in list(id2label.keys())]
 
         table_repartition = [
             ["Train"] + train_classes + [str(len(train_ds))],
@@ -141,25 +139,35 @@ class VisionDataset:
         return torch.utils.data.Subset(train_ds, list(range(0,len(train_ds)))), torch.utils.data.Subset(test_ds, list(range(0,len(test_ds))))
 
     @staticmethod
-    def fromImageFolder(dataset:str, test_ratio=0.15, balanced=True, augmentation=False):
+    def fromImageFolder(dataset:str, test_ratio=0.15, balanced=True, augmentation=False, torch_vision=False, transform=transformTorchVision):
         
         # Create ImageFolder from path
-        dataset = ImageFolder(dataset)
+        if torch_vision == True:
+            dataset = ImageFolder(dataset, transform)
+        else:
+            dataset = ImageFolder(dataset)
 
         # Both way indexes
         label2id, id2label = VisionDataset.getConfig(dataset)
 
+        img = transforms.ToPILImage()(dataset[0][0])
+        img.save("test_pil.jpg")
+
         # Split
-        train, test = VisionDataset.splitDatasets(dataset,id2label, test_ratio, balanced, augmentation)
+        train, test = VisionDataset.splitDatasets(dataset, id2label, test_ratio, balanced, augmentation)
 
         return train, test, id2label, label2id
 
     @staticmethod
-    def fromImageFolders(train:str, test:str):
+    def fromImageFolders(train:str, test:str, torch_vision=False, transform=transformTorchVision):
         
         # Split
-        train = ImageFolder(train)
-        test  = ImageFolder(test)
+        if torch_vision == True:
+            train = ImageFolder(train, transform)
+            test  = ImageFolder(test, transform)
+        else:
+            train = ImageFolder(train)
+            test  = ImageFolder(test)
 
         # Both way indexes
         label2id, id2label = VisionDataset.getConfig(train)
